from datetime import datetime
from unittest import TestCase

from betfair_api_client.betfair_api_client.datamodel.market import Market
from betfair_api_client.betfair_api_client.datamodel.runner import Runner


class TestMarket(TestCase):

    def __init__(self, methodName='runTest'):
        super(TestMarket, self).__init__(methodName=methodName)
        self.firstTestRunner = Runner(runnerId=1234, runnerName='firstTestRunner', handicap=1.0)
        self.secondTestRunner = Runner(runnerId=5678, runnerName='secondTestRunner', handicap=-2.0)

    def setUp(self):
        super().setUp()
        self.testMarket = Market(marketId='1.17234', marketName='testMarket', marketStartTime=datetime(year=1970, month=1, day=1))

    def test_add_runner(self):
        self.testMarket.add_runner(runner=self.firstTestRunner)
        self.assertEqual(len(self.testMarket.runners), 1)
        addedRunnerClass = list(self.testMarket.runners.values())[0]
        self.assertEqual(addedRunnerClass.runnerId, self.firstTestRunner.runnerId)

    def test_add_runner_duplicate(self):
        self.testMarket.add_runner(runner=self.firstTestRunner)
        self.testMarket.add_runner(runner=self.firstTestRunner)
        self.assertEqual(len(self.testMarket.runners), 1)
        addedRunnerClass = list(self.testMarket.runners.values())[0]
        self.assertEqual(addedRunnerClass.runnerId, self.firstTestRunner.runnerId)

    def test_add_runner_multiple(self):
        self.testMarket.add_runner(runner=self.firstTestRunner)
        self.testMarket.add_runner(runner=self.secondTestRunner)
        self.assertEqual(len(self.testMarket.runners), 2)
        addedRunnerClassIds = {runner.runnerId for runner in self.testMarket.runners.values()}
        self.assertTrue(addedRunnerClassIds == {self.firstTestRunner.runnerId, self.secondTestRunner.runnerId})

    def test_get_all_runners(self):
        self.testMarket.add_runner(runner=self.firstTestRunner)
        self.testMarket.add_runner(runner=self.secondTestRunner)
        allRunners = self.testMarket.get_all_runners()
        self.assertEqual(len(allRunners), 2)
        addedRunnerClassIds = {runner.runnerId for runner in allRunners}
        self.assertTrue(addedRunnerClassIds == {self.firstTestRunner.runnerId, self.secondTestRunner.runnerId})





class RunnerPrice:

    def __init__(self, betType, price, size):
        super().__init__()
        self.betType = betType
        self.price = price
        self.size = size

    def __str__(self):
        return f'betType: {self.betType}, price: {self.price}, size: {self.size}'

    def __repr__(self):
        return self.__str__()

    def __hash__(self):
        return hash((self.betType, self.price, self.size))

    def __eq__(self, other):
        return self.__hash__() == other.__hash__()




class MarketTypes:

    MATCH_ODDS = 'MATCH_ODDS'
    DRAW_NO_BET = 'DRAW_NO_BET'
    FULLTIME_SCORELINE = 'CORRECT_SCORE'
    HALFTIME_SCORELINE = 'HALF_TIME_SCORE'
    BOTH_TEAMS_TO_SCORE = 'BOTH_TEAMS_TO_SCORE'
    OVER_UNDER_05 = 'OVER_UNDER_05'
    OVER_UNDER_15 = 'OVER_UNDER_15'
    OVER_UNDER_25 = 'OVER_UNDER_25'
    OVER_UNDER_35 = 'OVER_UNDER_35'
    OVER_UNDER_45 = 'OVER_UNDER_45'
    OVER_UNDER_55 = 'OVER_UNDER_55'
    HALFTIME_FULLTIME = 'HALF_TIME_FULL_TIME'

    ALL = [
        MATCH_ODDS, DRAW_NO_BET, FULLTIME_SCORELINE, HALFTIME_SCORELINE, BOTH_TEAMS_TO_SCORE,
        OVER_UNDER_05, OVER_UNDER_15, OVER_UNDER_25, OVER_UNDER_35, OVER_UNDER_45, OVER_UNDER_55,
        HALFTIME_FULLTIME
    ]



from datetime import datetime

from .competition import Competition
from .market import Market


class Event:

    def __init__(self, eventId: int, eventName: str, eventDate: datetime, competition: Competition, countryCode: str):
        super().__init__()
        self.eventId = eventId
        self.eventName = eventName
        self.eventDate = eventDate
        self.competition = competition
        self.countryCode = countryCode
        self.markets = {}

    def __str__(self):
        return f'{self.eventName} ({self.eventDate}) - {self.competition.competitionName} ({self.countryCode})'

    def __repr__(self):
        return self.__str__()

    def add_market(self, market: Market):
        if market.marketId not in self.markets:
            self.markets[market.marketId] = market

    def get_all_markets(self):
        return list(self.markets.values())



import json
import logging
import requests
import urllib
import urllib.request
import urllib.error

from datetime import datetime
from datetime import timedelta
from typing import List
from warnings import warn

from .datamodel.bet_types import BetTypes
from .datamodel.competition import Competition
from .datamodel.event import Event
from .datamodel.exceptions import BetfairException
from .datamodel.market import Market
from .datamodel.runner import Runner
from .datamodel.runner_price import RunnerPrice


class BetfairApiClient:

    ACCOUNT_ENDPOINT = 'https://api.betfair.com/exchange/account/json-rpc/v1'
    BETTING_ENDPOINT = 'https://api.betfair.com/exchange/betting/json-rpc/v1'
    BETFAIR_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'

    def __init__(self, username: str, password: str, apiKey: str, clientCertificatePath: str, certificateKeyPath: str):
        """
        Client for non-interactive connections to the betfair API.
        Non-interactive (bot) logins require self-signed certificates: https://docs.developer.betfair.com/display/1smk3cen4v3lu3yomq5qye0ni/Non-Interactive+%28bot%29+login

        :param username: (str)
        :param password: (str)
        :param apiKey: (str)
        :param clientCertificatePath: (str)  Path to self-signed client certificate.
        :param certificateKeyPath: (str)  Path to self-signed client certificate key.
        """
        self.username = username
        self.password = password
        self.apiKey = apiKey
        self.clientCertificatePath = clientCertificatePath
        self.certificateKeyPath = certificateKeyPath
        self.sessionToken = None
        self.login()

    def login(self):
        response = requests.post(
            url='https://identitysso-cert.betfair.com/api/certlogin',
            data=f'username={self.username}&password={self.password}',
            cert=(self.clientCertificatePath, self.certificateKeyPath),
            headers={'X-Application': self.apiKey, 'Content-Type': 'application/x-www-form-urlencoded'}
        )
        jsonResponse = response.json()
        logging.info(msg=f"Betfair client login status: {jsonResponse['loginStatus']}")
        if response.status_code == 200:
            self.sessionToken = jsonResponse['sessionToken']
        else:
            logging.exception(msg='Request failed.')
            logging.exception(msg=jsonResponse['loginStatus'])
            raise Exception(jsonResponse['loginStatus'])

    def send_heartbeat(self):
        response = requests.post(
            url='https://identitysso.betfair.com/api/keepAlive',
            headers={'X-Authentication': self.sessionToken, 'Accept': 'application/json'}
        )
        logging.info(msg=response.json()['status'])
        return response

    def _call_api(self, jsonrpcRequest: dict, endpointURL: str):
        try:
            request = urllib.request.Request(
                url=endpointURL,
                data=json.dumps(jsonrpcRequest).encode('utf-8'),
                headers={'X-Application': self.apiKey, 'X-Authentication': self.sessionToken, 'content-type': 'application/json'}
            )
            response = urllib.request.urlopen(request).read()
            decodedResponse = json.loads(response.decode('utf-8'))
            if 'result' in decodedResponse:
                return decodedResponse['result']
            return decodedResponse
        except urllib.error.URLError as ex:
            logging.exception(msg=ex)
            raise ex

    def check_balance(self):
        balanceRequest = {'jsonrpc': '2.0', 'method': 'AccountAPING/v1.0/getAccountFunds'}
        return self._call_api(jsonrpcRequest=balanceRequest, endpointURL=self.ACCOUNT_ENDPOINT)

    def list_competitions(self, countryCodes: List[str], sportTypeIds: List[int]):
        competitionsRequest = {
            "params": {
                "filter": {
                    "eventTypeIds": sportTypeIds,
                    'marketCountries': countryCodes,
                }
            },
            "jsonrpc": "2.0",
            "method": "SportsAPING/v1.0/listCompetitions",
            "id": 1
        }
        rawCompetitionData = self._call_api(jsonrpcRequest=competitionsRequest, endpointURL=self.BETTING_ENDPOINT)
        return [Competition(competitionId=int(competition['competition']['id']), competitionName=competition['competition']['name']) for competition in rawCompetitionData]

    def get_coming_events(self, sportTypeId: int, marketTypes: List[str], countryCodes: List[str] = None, textQuery: str = None, competitionIds: List[int] = None, daysAhead: int = 7):
        parameterDictionary = {
            'eventTypeIds': [sportTypeId],
            'marketTypeCodes': marketTypes,
            'marketStartTime': {
                'from': datetime.now().strftime('%Y-%m-%d'),
                'to': (datetime.now() + timedelta(days=daysAhead + 1)).strftime('%Y-%m-%d')
            }
        }
        if countryCodes is not None:
            parameterDictionary['marketCountries'] = countryCodes
        if textQuery is not None:
            parameterDictionary['textQuery'] = textQuery
        if competitionIds is not None:
            parameterDictionary['competitionIds'] = [int(competitionId) for competitionId in competitionIds]
        catalogueRequest = {
            'jsonrpc': '2.0',
            'method': 'SportsAPING/v1.0/listMarketCatalogue',
            'params': {
                'filter': parameterDictionary,
                'maxResults': 1000,
                'marketProjection': ['COMPETITION', 'EVENT', 'EVENT_TYPE', 'RUNNER_DESCRIPTION', 'RUNNER_METADATA', 'MARKET_START_TIME']
            },
            'id': 1
        }
        rawMarketsData = self._call_api(jsonrpcRequest=catalogueRequest, endpointURL=self.BETTING_ENDPOINT)
        if 'error' in rawMarketsData:
            raise BetfairException(rawMarketsData['error']['message'])
        processedEvents = self._process_raw_markets_data(rawMarketsData=rawMarketsData)
        return processedEvents

    def _process_raw_markets_data(self, rawMarketsData: List[dict]):
        processedEvents = {}
        for rawMarket in rawMarketsData:
            eventId = rawMarket['event']['id']
            if eventId not in processedEvents:
                competition = Competition(competitionId=int(rawMarket['competition']['id']), competitionName=rawMarket['competition']['name'])
                processedEvents[eventId] = Event(
                    eventId=int(eventId),
                    eventName=rawMarket['event']['name'],
                    eventDate=datetime.strptime(rawMarket['event']['openDate'], self.BETFAIR_DATETIME_FORMAT),
                    competition=competition,
                    countryCode=rawMarket['event']['countryCode']
                )
            market = Market(
                marketId=rawMarket['marketId'],
                marketName=rawMarket['marketName'],
                marketStartTime=datetime.strptime(rawMarket['marketStartTime'], self.BETFAIR_DATETIME_FORMAT)
            )
            for selection in rawMarket['runners']:
                market.add_runner(
                    runner=Runner(runnerId=int(selection['selectionId']), runnerName=selection['runnerName'], handicap=selection['handicap'])
                )
            processedEvents[eventId].add_market(market=market)
        return list(processedEvents.values())

    def update_prices_for_events(self, events: List[Event]):
        for i, event in enumerate(events):
            bookRequest = {
                'jsonrpc': '2.0',
                'method': 'SportsAPING/v1.0/listMarketBook',
                'params': {
                    'marketIds': list(event.markets.keys()),
                    'priceProjection': {'priceData': ['EX_BEST_OFFERS']},
                    'maxResults': '1000'
                },
                'id': 1
            }
            recentMarketData = self._call_api(jsonrpcRequest=bookRequest, endpointURL=self.BETTING_ENDPOINT)
            if i == 0 and len(recentMarketData):
                if recentMarketData[0]['isMarketDataDelayed']:
                    warn(message='Market data is delayed.  You may need to upgrade your Betfair Developer account to access live data.')
            for marketData in recentMarketData:
                marketIdUpdate = str(marketData['marketId'])
                for runnerInfo in marketData['runners']:
                    runnerUpdateId = int(runnerInfo['selectionId'])
                    backPrices = [RunnerPrice(betType=BetTypes.BACK, price=p['price'], size=p['size']) for p in runnerInfo['ex'][BetTypes.BACK]]
                    layPrices = [RunnerPrice(betType=BetTypes.LAY, price=p['price'], size=p['size']) for p in runnerInfo['ex'][BetTypes.LAY]]
                    # TODO(Mike): this is horrible
                    event.markets[marketIdUpdate].runners[runnerUpdateId].update_back_odds(availableToBack=backPrices)
                    event.markets[marketIdUpdate].runners[runnerUpdateId].update_lay_odds(availableToLay=layPrices)
            return events

    def place_bet(self, market: Market, runner: Runner, oddsToPlace: float, side: str, betSize: float):
        placeOrderRequest = {
            'jsonrpc': '2.0',
            'method': 'SportsAPING/v1.0/placeOrders',
            'params': {
                'marketId': market.marketId,
                'instructions': [{
                    'selectionId': runner.runnerId,
                    'side': side,
                    'orderType': 'LIMIT',
                    'limitOrder': {
                           'size': float(betSize),
                           'price': float(oddsToPlace),
                           'persistenceType': 'LAPSE'
                       }
                   }]
            },
            'id': 1
        }
        response = self._call_api(jsonrpcRequest=placeOrderRequest, endpointURL=self.BETTING_ENDPOINT)
        if response['status'] == 'FAILURE':
            logging.exception(msg=response)
            raise BetfairException(response['errorCode'])
        return response




class Competition:

    def __init__(self, competitionName: str, competitionId: int):
        self.competitionName = competitionName.strip()
        self.competitionId = int(competitionId)

    def __repr__(self):
        return f'{self.competitionName} ({self.competitionId})'

    def __str__(self):
        return self.__repr__()

    def __eq__(self, other):
        return self.__hash__() == other.__hash__()

    def __hash__(self):
        return hash((self.competitionName, self.competitionId))



from datetime import datetime
from unittest import TestCase

from betfair_api_client.betfair_api_client.datamodel.competition import Competition
from betfair_api_client.betfair_api_client.datamodel.event import Event
from betfair_api_client.betfair_api_client.datamodel.market import Market


class TestEvent(TestCase):

    def __init__(self, methodName='runTest'):
        super(TestEvent, self).__init__(methodName=methodName)
        self.firstTestMarket = Market(marketId='1.234567', marketName='firstTestMarket', marketStartTime=datetime(year=1970, month=1, day=1))
        self.secondTestMarket = Market(marketId='7.654321', marketName='secondTestMarket', marketStartTime=datetime(year=1980, month=12, day=12))

    def setUp(self):
        super().setUp()
        self.testEvent = Event(
            eventId=97531,
            eventName='testEvent',
            eventDate=datetime(year=1990, month=6, day=6),
            competition=Competition(competitionName='testCompetition', competitionId=24),
            countryCode='GB'
        )

    def test_add_market_single(self):
        self.testEvent.add_market(market=self.firstTestMarket)
        self.assertEqual(len(self.testEvent.markets), 1)
        addedMarketClass = list(self.testEvent.markets.values())[0]
        self.assertEqual(addedMarketClass.marketId, self.firstTestMarket.marketId)

    def test_add_market_duplicate(self):
        self.testEvent.add_market(market=self.firstTestMarket)
        self.testEvent.add_market(market=self.firstTestMarket)
        self.assertEqual(len(self.testEvent.markets), 1)
        addedMarketClass = list(self.testEvent.markets.values())[0]
        self.assertEqual(addedMarketClass.marketId, self.firstTestMarket.marketId)

    def test_add_market_multiple(self):
        self.testEvent.add_market(market=self.firstTestMarket)
        self.testEvent.add_market(market=self.secondTestMarket)
        self.assertEqual(len(self.testEvent.markets), 2)
        addedMarketClassIds = {market.marketId for market in self.testEvent.markets.values()}
        self.assertTrue(addedMarketClassIds == {self.firstTestMarket.marketId, self.secondTestMarket.marketId})

    def test_get_all_markets(self):
        self.testEvent.add_market(market=self.firstTestMarket)
        self.testEvent.add_market(market=self.secondTestMarket)
        allMarkets = self.testEvent.get_all_markets()
        self.assertEqual(len(allMarkets), 2)
        addedMarketClassIds = {market.marketId for market in allMarkets}
        self.assertTrue(addedMarketClassIds == {self.firstTestMarket.marketId, self.secondTestMarket.marketId})



from datetime import datetime

from .runner import Runner


class Market:

    def __init__(self, marketId: str, marketName: str, marketStartTime: datetime):
        self.marketId = marketId
        self.marketName = marketName
        self.marketStartTime = marketStartTime
        self.runners = {}

    def __repr__(self):
        return f'Market: "{self.marketName}".  Starts at: {self.marketStartTime}'

    def __str__(self):
        return self.__repr__()

    def add_runner(self, runner: Runner):
        if runner.runnerId not in self.runners:
            self.runners[runner.runnerId] = runner

    def get_all_runners(self):
        return list(self.runners.values())




class ErrorCodes:

    JSON_PARSE_FAILURE = 'DSC-0008'
    INVALID_FORMAT_FOR_PARAMETER = 'DSC-0009'
    SECURITY_EXCEPTION = 'DSC-0015'
    PARAMETER_NOT_DEFINED = 'DSC-0018'
    TIMEOUT = 'DSC-0019'
    NO_SUCH_OPERATION = 'DSC-0021'
    NO_SUCH_SERVICE = 'DSC-0023'
    DESERIALISATION_FAILURE = 'DSC-0024'
    UNKNOWN_CALLER = 'DSC-0034'
    UNRECOGNIZED_CREDENTIALS = 'DSC-0035'
    INVALID_CREDENTIALS = 'DSC-0036'
    SUBSCRIPTION_REQUIRED = 'DSC-0037'
    OPERATION_FORBIDDEN = 'DSC-0038'
    TOO_MUCH_DATA = 'ANGX-0001'
    NOT_ENOUGH_FUNDS = 'INSUFFICIENT_FUNDS'


class BetfairException(BaseException):

    def __init__(self, betfairExceptionCode):

        if betfairExceptionCode == ErrorCodes.JSON_PARSE_FAILURE:
            raise JsonParseFailure

        elif betfairExceptionCode == ErrorCodes.INVALID_FORMAT_FOR_PARAMETER:
            raise InvalidFormatForParameter

        elif betfairExceptionCode == ErrorCodes.SECURITY_EXCEPTION:
            raise SecurityException

        elif betfairExceptionCode == ErrorCodes.PARAMETER_NOT_DEFINED:
            raise ParameterNotDefined

        elif betfairExceptionCode == ErrorCodes.TIMEOUT:
            raise TimeoutException

        elif betfairExceptionCode == ErrorCodes.NO_SUCH_OPERATION:
            raise NoSuchOperation

        elif betfairExceptionCode == ErrorCodes.NO_SUCH_SERVICE:
            raise NoSuchService

        elif betfairExceptionCode == ErrorCodes.DESERIALISATION_FAILURE:
            raise DeserializationError

        elif betfairExceptionCode == ErrorCodes.UNKNOWN_CALLER:
            raise UnknownCaller

        elif betfairExceptionCode == ErrorCodes.UNRECOGNIZED_CREDENTIALS:
            raise UnrecognizedCredentials

        elif betfairExceptionCode == ErrorCodes.INVALID_CREDENTIALS:
            raise InvalidCredentials

        elif betfairExceptionCode == ErrorCodes.SUBSCRIPTION_REQUIRED:
            raise SubscriptionRequired

        elif betfairExceptionCode == ErrorCodes.OPERATION_FORBIDDEN:
            raise OperationForbidden

        elif betfairExceptionCode == ErrorCodes.TOO_MUCH_DATA:
            raise TooMuchData

        elif betfairExceptionCode == ErrorCodes.NOT_ENOUGH_FUNDS:
            raise NotEnoughFunds


class JsonParseFailure(Exception):
    pass


class InvalidFormatForParameter(Exception):
    pass


class SecurityException(Exception):
    pass


class ParameterNotDefined(Exception):
    pass


class TimeoutException(Exception):
    pass


class NoSuchOperation(Exception):
    pass


class NoSuchService(Exception):
    pass


class DeserializationError(Exception):
    pass


class UnknownCaller(Exception):
    pass


class UnrecognizedCredentials(Exception):
    pass


class InvalidCredentials(Exception):
    pass


class SubscriptionRequired(Exception):
    pass


class OperationForbidden(Exception):
    pass


class TooMuchData(Exception):
    pass


class NotEnoughFunds(Exception):
    pass



import setuptools

setuptools.setup(
    name='betfair_api_client',
    packages=setuptools.find_packages(),
    version='0.1',
    description='Client for connecting to the Betfair Odds API and fetch betting market info',
    author='Michael Doran',
    author_email='mikrdoran@gmail.com',
    url='https://github.com/miksyr/betfair_api_client',
    download_url='https://github.com/miksyr/betfair_api_client/archive/v_01.tar.gz',
    keywords=['betfair', 'betfair odds api', 'sports betting'],
    install_requires=['requests==2.24.0'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)



from typing import List

from .bet_types import BetTypes
from .runner_price import RunnerPrice


class Runner:

    def __init__(self, runnerId: int, runnerName: str, handicap: float):
        super().__init__()
        self.runnerId = runnerId
        self.runnerName = runnerName.strip()
        self.handicap = float(handicap)
        self.availableToBack = []
        self.availableToLay = []

    def __str__(self):
        return f'{self.runnerName} ({self.runnerId})'

    def __repr__(self):
        return self.__str__()

    def get_best_back_price(self):
        if len(self.availableToBack) == 0:
            return RunnerPrice(betType=BetTypes.BACK, price=0, size=0)
        return sorted(self.availableToBack, key=lambda x: x.price, reverse=True)[0]

    def get_best_lay_price(self):
        if len(self.availableToLay) == 0:
            return RunnerPrice(betType=BetTypes.LAY, price=0, size=0)
        return sorted(self.availableToLay, key=lambda x: x.price)[0]

    def get_all_available_runner_prices(self):
        return self.availableToBack + self.availableToLay

    def update_back_odds(self, availableToBack: List[RunnerPrice]):
        self.availableToBack = availableToBack

    def update_lay_odds(self, availableToLay: List[RunnerPrice]):
        self.availableToLay = availableToLay



from unittest import TestCase

from betfair_api_client.betfair_api_client.datamodel.bet_types import BetTypes
from betfair_api_client.betfair_api_client.datamodel.runner import Runner
from betfair_api_client.betfair_api_client.datamodel.runner_price import RunnerPrice


class TestRunner(TestCase):

    def __init__(self, methodName='runTest'):
        super(TestRunner, self).__init__(methodName=methodName)
        self.runnerName = 'testRunner'
        self.runnerId = 12345
        self.betfairRunnerTableId = 100
        self.availableToBack = [RunnerPrice(betType=BetTypes.BACK, price=i, size=i * 100) for i in range(0, 10, 3)]
        self.bestAvailableToBack = RunnerPrice(betType=BetTypes.BACK, price=9, size=900)
        self.availableToLay = [RunnerPrice(betType=BetTypes.LAY, price=i, size=i * 100) for i in range(10, 20, 3)]
        self.bestAvailableToLay = RunnerPrice(betType=BetTypes.LAY, price=10, size=1000)

    def setUp(self):
        super().setUp()
        self.noOddsRunner = Runner(runnerName=self.runnerName, runnerId=self.runnerId, handicap=0.0)
        self.oddsRunner = Runner(runnerName=self.runnerName, runnerId=self.runnerId, handicap=0.0)
        self.oddsRunner.update_back_odds(availableToBack=self.availableToBack)
        self.oddsRunner.update_lay_odds(availableToLay=self.availableToLay)

    def test_get_best_back_price(self):
        bestBackPrice = self.oddsRunner.get_best_back_price()
        self.assertEqual(bestBackPrice, self.bestAvailableToBack)

    def test_get_best_lay_price(self):
        bestLayPrice = self.oddsRunner.get_best_lay_price()
        self.assertEqual(bestLayPrice, self.bestAvailableToLay)

    def test_get_all_available_runner_prices(self):
        allPrices = self.oddsRunner.get_all_available_runner_prices()
        self.assertEqual(set(allPrices), set(self.availableToBack + self.availableToLay))

    def test_update_back_odds(self):
        self.noOddsRunner.update_back_odds(availableToBack=self.availableToBack)
        bestBackPrice = self.noOddsRunner.get_best_back_price()
        self.assertEqual(bestBackPrice, self.bestAvailableToBack)

    def test_update_lay_odds(self):
        self.noOddsRunner.update_lay_odds(availableToLay=self.availableToLay)
        bestLayPrice = self.noOddsRunner.get_best_lay_price()
        self.assertEqual(bestLayPrice, self.bestAvailableToLay)



import os
from datetime import datetime
from unittest import TestCase

from betfair_api_client.betfair_api_client import BetfairApiClient
from betfair_api_client.betfair_api_client.datamodel.competition import Competition
from betfair_api_client.betfair_api_client.datamodel.event import Event
from betfair_api_client.betfair_api_client.datamodel.exceptions import TooMuchData
from betfair_api_client.betfair_api_client.datamodel.market import Market
from betfair_api_client.betfair_api_client.datamodel.market_types import MarketTypes
from betfair_api_client.betfair_api_client.datamodel.runner import Runner
from betfair_api_client.betfair_api_client.datamodel.runner_price import RunnerPrice


TEST_COMPETITION = Competition(competitionId=12117172, competitionName='Australian A-League')
TEST_EVENT = Event(
    eventId=29939806,
    eventName='Western Sydney Wanderers v Perth Glory',
    eventDate=datetime(year=2020, month=8, day=4, hour=9, minute=30, second=0),
    competition=TEST_COMPETITION,
    countryCode='AU'
)
TEST_MARKET = Market(marketId='1.171796736', marketName='Match Odds', marketStartTime=datetime(year=2020, month=8, day=4, hour=9, minute=30, second=0))
FIRST_TEST_RUNNER = Runner(runnerId=6480414, runnerName='Western Sydney Wanderers', handicap=0.0)
SECOND_TEST_RUNNER = Runner(runnerId=370132, runnerName='Perth Glory', handicap=-2.0)
THIRD_TEST_RUNNER = Runner(runnerId=58805, runnerName='The Draw', handicap=1.0)
ALL_TEST_RUNNERS = [FIRST_TEST_RUNNER, SECOND_TEST_RUNNER, THIRD_TEST_RUNNER]


class TestBetfairClient(TestCase):

    def __init__(self, methodName='runTest'):
        super(TestBetfairClient, self).__init__(methodName=methodName)
        self.betfairClient = BetfairApiClient(
            username=os.environ['BETFAIR_USERNAME'],
            password=os.environ['BETFAIR_PASSWORD'],
            apiKey=os.environ['BETFAIR_API_KEY'],
            clientCertificatePath=os.environ['BETFAIR_CLIENT_CERT'],
            certificateKeyPath=os.environ['BETFAIR_CLIENT_CERT_KEY']
        )
        self.rawMatchOddsData = [
            {
                'marketId': TEST_MARKET.marketId,
                'marketName': TEST_MARKET.marketName,
                'marketStartTime': TEST_MARKET.marketStartTime.strftime(BetfairApiClient.BETFAIR_DATETIME_FORMAT),
                'runners': [
                    {
                        'selectionId': FIRST_TEST_RUNNER.runnerId,
                        'runnerName': FIRST_TEST_RUNNER.runnerName,
                        'handicap': FIRST_TEST_RUNNER.handicap,
                    },
                    {
                        'selectionId': SECOND_TEST_RUNNER.runnerId,
                        'runnerName': SECOND_TEST_RUNNER.runnerName,
                        'handicap': SECOND_TEST_RUNNER.handicap,
                    },
                    {
                        'selectionId': THIRD_TEST_RUNNER.runnerId,
                        'runnerName': THIRD_TEST_RUNNER.runnerName,
                        'handicap': THIRD_TEST_RUNNER.handicap,
                    }
                ],
                'eventType': {'id': '1', 'name': 'Soccer'},
                'competition': {'id': str(TEST_COMPETITION.competitionId), 'name': TEST_COMPETITION.competitionName},
                'event': {
                    'id': str(TEST_EVENT.eventId),
                    'name': TEST_EVENT.eventName,
                    'countryCode': TEST_EVENT.countryCode,
                    'openDate': TEST_EVENT.eventDate.strftime(self.betfairClient.BETFAIR_DATETIME_FORMAT),
                }
            }
        ]
        self.exampleProcessedEvent = TEST_EVENT
        for runner in ALL_TEST_RUNNERS:
            TEST_MARKET.add_runner(runner=runner)
        self.exampleProcessedEvent.add_market(market=TEST_MARKET)

    def test_login(self):
        self.assertIsNotNone(self.betfairClient.sessionToken)
        self.assertNotEqual(self.betfairClient.sessionToken, '')

    def test_send_heartbeat(self):
        response = self.betfairClient.send_heartbeat()
        self.assertTrue(response.ok)

    def test_check_balance(self):
        dictResponse = self.betfairClient.check_balance()
        self.assertTrue('availableToBetBalance' in dictResponse)

    def test_list_competitions(self):
        results = self.betfairClient.list_competitions(countryCodes=['GB'], sportTypeIds=[1])
        self.assertTrue(Competition(competitionId=10932509, competitionName='English Premier League') in results)

    def test_get_coming_events(self):
        try:
            comingEvents = self.betfairClient.get_coming_events(sportTypeId=1, marketTypes=[MarketTypes.MATCH_ODDS], daysAhead=1)
        except TooMuchData:
            try:
                comingEvents = self.betfairClient.get_coming_events(sportTypeId=1, countryCodes=['GB'], marketTypes=[MarketTypes.MATCH_ODDS])
            except TooMuchData:
                comingEvents = self.betfairClient.get_coming_events(sportTypeId=1, marketTypes=[MarketTypes.MATCH_ODDS], daysAhead=0)
        self.assertTrue(isinstance(comingEvents, list))
        self.assertTrue(len(comingEvents) > 0)
        self.assertTrue(isinstance(comingEvents[0], Event))

    def test_process_raw_markets_data(self):
        processedEvent = self.betfairClient._process_raw_markets_data(rawMarketsData=self.rawMatchOddsData)[0]
        self.assertEqual(processedEvent.eventId, self.exampleProcessedEvent.eventId)
        self.assertEqual(processedEvent.eventName, self.exampleProcessedEvent.eventName)
        self.assertEqual(processedEvent.eventDate, self.exampleProcessedEvent.eventDate)
        processedMarket = processedEvent.get_all_markets()[0]
        self.assertEqual(processedMarket.marketId, TEST_MARKET.marketId)
        self.assertEqual(processedMarket.marketName, TEST_MARKET.marketName)
        self.assertEqual(processedMarket.marketStartTime, TEST_MARKET.marketStartTime)
        processedRunners = processedMarket.get_all_runners()
        self.assertEqual({r.runnerId for r in processedRunners}, {r.runnerId for r in ALL_TEST_RUNNERS})
        self.assertEqual({r.runnerName for r in processedRunners}, {r.runnerName for r in ALL_TEST_RUNNERS})
        self.assertEqual({r.handicap for r in processedRunners}, {r.handicap for r in ALL_TEST_RUNNERS})

    def test_update_prices_for_events(self):
        try:
            comingEvents = self.betfairClient.get_coming_events(sportTypeId=1, marketTypes=[MarketTypes.MATCH_ODDS], daysAhead=1)
        except TooMuchData:
            try:
                comingEvents = self.betfairClient.get_coming_events(sportTypeId=1, countryCodes=['GB'], marketTypes=[MarketTypes.MATCH_ODDS], daysAhead=1)
            except TooMuchData:
                comingEvents = self.betfairClient.get_coming_events(sportTypeId=1, marketTypes=[MarketTypes.MATCH_ODDS], daysAhead=0)
        updatedEvents = self.betfairClient.update_prices_for_events(events=comingEvents)
        exampleMarket = updatedEvents[0].get_all_markets()[0]
        marketRunners = exampleMarket.get_all_runners()
        for runner in marketRunners:
            bestBackPrice = runner.get_best_back_price()
            self.assertTrue(isinstance(bestBackPrice, RunnerPrice))
            self.assertTrue(bestBackPrice.price > 0)
            self.assertTrue(bestBackPrice.size > 0)
            bestLayPrice = runner.get_best_lay_price()
            self.assertTrue(isinstance(bestLayPrice, RunnerPrice))
            self.assertTrue(bestLayPrice.price > 0)
            self.assertTrue(bestLayPrice.size > 0)
            self.assertTrue(len(runner.get_all_available_runner_prices()) > 0)
